package ue810_UserDb.controller;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import ue810_UserDb.model.User;
import ue810_UserDb.model.UserList;

/****************************************************************************************************
 * Klasse Tbl_User
 * Diese Klasse stellt Funktionen zur Interaktion mit einer Datenbank-Tabelle "users" bereit.
 * Sie ermöglicht CRUD-Operationen (Create, Read, Update, Delete) und dient als Vermittler
 * zwischen der Datenbank und der Anwendung.
 *
 * @author Prof. Gaisberger
 * @version 1.0
 ****************************************************************************************************
 */

public class Tbl_User {

    // ---------------------------------------------------------------------------------------------
    // Instanzvariablen/Klassenvariablen
    // ---------------------------------------------------------------------------------------------
    private Connection dbConnection; // Verbindung zur Datenbank

    // ---------------------------------------------------------------------------------------------
    // Konstruktor
    /** ----------------------------------------------------------------------------------------<br>
     * Initialisiert eine neue Tbl_User-Instanz mit einer bestehenden Datenbankverbindung.
     *
     * @param aDbController Ein DbController-Objekt, das die Datenbankverbindung bereitstellt.
     */
    public Tbl_User(DbController aDbController) {
        dbConnection = aDbController.getDbConnection();
    }

    // ---------------------------------------------------------------------------------------------
    // Methoden
    /** ----------------------------------------------------------------------------------------<br>
     * Sucht einen Benutzer anhand seiner ID.
     *
     * @param id Die eindeutige ID des gesuchten Benutzers.
     * @return Ein User-Objekt oder null, falls kein Benutzer gefunden wurde.
     */
    public User selectById(int id) {
        String sql = "SELECT id, name, email FROM users WHERE id = ?";

        try {
            PreparedStatement pstmt = dbConnection.prepareStatement(sql);
            pstmt.setInt(1, id);
            ResultSet resultset = pstmt.executeQuery();

            if (resultset.next()) {
                return new User(resultset.getInt("id"), resultset.getString("name"), resultset.getString("email"));
            }
        } catch (SQLException e) {
            e.printStackTrace(); // Fehlerbehandlung
        }

        return null;
    }
    
    /** ----------------------------------------------------------------------------------------<br>
     * Sucht nach Benutzern anhand ihres Namens.
     *
     * @param aName Der Name oder ein Namensmuster der gesuchten Benutzer.
     * @return Eine UserList mit allen passenden Benutzern oder null im Fehlerfall.
     */
    public UserList selectByName(String aName) {
        String sql = "SELECT id, name, email FROM users WHERE name LIKE ?";

        try {
            PreparedStatement pstmt = dbConnection.prepareStatement(sql);
						// SQLLite ist nur FORWARDONLY
  						// ORACEL:  ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY
    
            pstmt.setString(1, aName);
            ResultSet byResultset = pstmt.executeQuery();

            return new UserList(byResultset);
        } catch (SQLException e) {
            e.printStackTrace(); // Fehlerbehandlung
        }
        return null;
    }

    
     /** ----------------------------------------------------------------------------------------<br>
     * Aktualisiert die Daten eines Benutzers in der Datenbank.
     *
     * @param aUser Ein User-Objekt mit den aktualisierten Daten.
     */
    public void update(User aUser) {
        String sql = "UPDATE users SET name = ?, email = ? WHERE id = ?";

        try (PreparedStatement pstmt = dbConnection.prepareStatement(sql)) {
            pstmt.setString(1, aUser.getName());
            pstmt.setString(2, aUser.getEmail());
            pstmt.setInt(3, aUser.getId());

            pstmt.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace(); // Fehlerbehandlung
        }
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Erstellt die Tabelle "users", falls sie nicht existiert.
     */
    public void createTable() {
        String createTableSQL = "CREATE TABLE IF NOT EXISTS users (" +
                                "id INTEGER PRIMARY KEY AUTOINCREMENT," +
                                "name TEXT NOT NULL," +
                                "email TEXT NOT NULL" +
                                ");";

        try (Statement stmt = dbConnection.createStatement()) {
            stmt.execute(createTableSQL);
            System.out.println("Tabelle erstellt.");
        } catch (SQLException e) {
            e.printStackTrace(); // Fehlerbehandlung
        }
    }

    /** ----------------------------------------------------------------------------------------<br>
     * Fügt Demodaten in die Tabelle ein.
     */
    public void insertData() {
        String insertDataSQL = "INSERT INTO users (name, email) VALUES " +
                               "('John Doe', 'john.doe@example.com')," +
                               "('Jane Smith', 'jane.smith@example.com')," +
                               "('Sam Won', 'sam.won@example.com')," +
                               "('Anne Won', 'anne.won@example.com');";

        try (Statement stmt = dbConnection.createStatement()) {
            stmt.execute(insertDataSQL);
            System.out.println("Daten eingefügt.");
        } catch (SQLException e) {
            e.printStackTrace(); // Fehlerbehandlung
        }
    }
    
    public void deleteData(User aUser)
    {
        String deleteDataSql = "DELETE FROM your_table_name WHERE your_column_name = ?"; // Adjust the table and column names 
 
        try (PreparedStatement pstmt = dbConnection.prepareStatement(deleteDataSql)) {
            pstmt.setString(1, aUser.getName());

            pstmt.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace(); // Fehlerbehandlung
        }
    } 
}
